#include <Types.h>
#include <PLStringFuncs.h>
#include <Memory.h>
#include <AppleTalk.h>
#include <GestaltEqu.h>
#include <time.h>
#include <errors.h>

#include "Common.h"

extern OSErr CallCompletion(short trapword, MPPPBPtr pb, long routine);
extern OSErr CallControl(short trapword, MPPPBPtr pb);

static MyGestaltPtr GetGestaltInfo(void)
{
	register MyGestaltPtr	pb;
	register OSErr			error;
	
	error = Gestalt(getGestalt(), (long *)&pb);
	if (error) pb = 0;
	return pb;
}

static NameLinkPtr GetRegisteredNames(void)
{
	register NameLinkPtr pb = nil;
	register MyGestaltPtr pc;
	
	pc = GetGestaltInfo();
	if (pc)
	{
		pb = pc->registeredNames;
	}
	return pb;
}

static OriginalLinkPtr GetMaskableNames(void)
{
	register OriginalLinkPtr	pb = nil;
	register MyGestaltPtr		pc;
	
	pc = GetGestaltInfo();
	if (pc)
	{
		pb = pc->trappedNames;
	}
	return pb;
}

/*
  Is the string one that we should mask out?
*/

static long StringIsMaskable(char* theString)
{
	register OriginalLinkPtr pb = GetMaskableNames();
	register char* maskString;
	register short result;

	if (!pb) return 0;
	
	maskString = pb->originalName;

	while (true)
	{
		result = maskString ? !PLstrcmp(theString, maskString) : false;
		if (result)
		{
			return pb->objectPrefs;
		}
		pb = pb->next;
		if (!pb) break;
		maskString = pb->originalName;
	}
	return 0;
}

/*
  Has the string already been masked out?
*/

static Boolean IsStringMasked(char* theString)
{
	register NameLinkPtr pb = GetRegisteredNames();
	register char* registeredName;
	register short result;

	if (!pb) return false;
	registeredName = pb->originalName;

	while (true)
	{
		result = registeredName ? !PLstrcmp(theString, registeredName) : false;
		if (result)
		{
			return true;
		}
		pb = pb->next;
		if (!pb) break;
		registeredName = pb->originalName;
	}
	return false;
}

/*
  This is made 4 quark: it tries to deregister a string,
  so if the string is already masked set the string passed
  in to the mask.
*/

static void SetStringToMask(char* theString)
{
	register NameLinkPtr pb = GetRegisteredNames();
	register char* registeredName;
	register short result;

	if (!pb) return;
	registeredName = pb->originalName;

	while (true)
	{
		result = registeredName ? !PLstrcmp(theString, registeredName) : false;
		if (result)
		{
			PLstrcpy(theString, pb->newString);
			return;
		}
		pb = pb->next;
		if (!pb) break;
		registeredName = pb->originalName;
	}
	return;
}

/*
  Munge the string
*/

static void Munge(char* theString)
{
	register short i;
	register char* j;
	register char k;
	
	j = theString;
	i = theString[0];
	while (i)
	{
		k = clock() + 1;
		if ((k == '') || (k == '*') || (k == 0) || (k == '=')) k++;	// no wildcards!
		theString[i--] = k;
	}
}

/*
  Insert the old & new strings into the mask list, & munge too
*/

static void MaskString(char* theString)
{
	register NameLinkPtr pb = GetRegisteredNames();
	register NameLinkPtr newLink;
	
	if (!pb) return;

	newLink = (NameLinkPtr) NewPtrSysClear(sizeof(NameLink));
	if (!newLink) return;
	PLstrcpy(newLink->originalName, theString);
	Munge(theString);
	PLstrcpy(newLink->newString, theString);

	while (true)
	{
		if (pb->next)
		{
			pb = pb->next;
		}
		else
		{
			pb->next = newLink;
			newLink->previous = pb;
			return;
		}
	}

}

/*
  removes an entry from the NameLink list
*/

static void RemoveEntry(char* theString)
{
	register NameLinkPtr pb = GetRegisteredNames();
	register char* str;

	if (!pb || !theString || !theString[0]) return;
	
	str = pb->newString;
	while (true)
	{
		if (str && str[0])
		if (!PLstrcmp(str, theString))
		{
			if (pb->previous)
			{
				(pb->previous)->next = pb->next;
			}
			if (pb->next)
			{
				(pb->next)->previous = pb->previous;
			}
			DisposePtr((Ptr)pb);
			return;
		}
		pb = pb->next;
		if (!pb) break;
		str = pb->newString;
	}

}

static void CheckRegister(char* typeName)
{
	if (StringIsMaskable(typeName))				// should we worry about wiping the type?
	{
		if (!IsStringMasked(typeName))
		{
			MaskString(typeName);
		}
	}
}

static void RestoreString(char* theString)
{
	register NameLinkPtr pb = GetRegisteredNames();
	register char* registeredName;
	register short result;

	if (!pb) return;
	registeredName = pb->newString;

	while (true)
	{
		result = registeredName ? !PLstrcmp(theString, registeredName) : false;
		if (result)
		{
			PLstrcpy(theString, pb->newString);
			return;
		}
		pb = pb->next;
		if (!pb) break;
		registeredName = pb->newString;
	}
}

OSErr DoNewRemove(short trapword, MPPPBPtr pb)
{
	register OSErr error;
	register char *objName, *typeName;
	register Boolean async, objGood = false, typeGood = false, goAsync = false;
	register long objectPrefs = 0, typePrefs = 0;
	
	async = trapword & 0x0400;
	objName = ((char *)pb->NBPentityPtr);
	typeName = objName + objName[0] + 1;
	
	if (objName && objName[0])
	{
		if ((objectPrefs = StringIsMaskable(objName)) && IsStringMasked(objName))
		{
			if (removeEnabled(objectPrefs))
			{
				SetStringToMask(objName);
				goAsync = removeAsync(objectPrefs);
				objGood = true;
			}
		}
		RemoveEntry(objName);
	}

	if (typeName && typeName[0])
	{
		if ((typePrefs = StringIsMaskable(typeName)) && IsStringMasked(typeName))
		{
			if (removeEnabled(typePrefs))
			{
				SetStringToMask(typeName);
				goAsync |= removeAsync(typePrefs);
				typeGood = true;
			}
		}
		RemoveEntry(typeName);
	}

#if 1
	error = CallControl(trapword, pb);
#else
	error = CallControl(async ? (goAsync ? trapword : trapword & 0x0400) : trapword, pb);

	if (async && !goAsync) pb->MPPioResult = error;
	
	if (objGood && !goAsync) RestoreString(objName);
	if (typeGood && !goAsync) RestoreString(typeName);
	
	if (!goAsync && async && pb->MPPioCompletion)
	{
		CallCompletion(trapword, pb, (long) pb->MPPioCompletion);
	}
#endif
	return error;
}

OSErr DoNewRegister(short trapword, MPPPBPtr pb)
{
	register OSErr error;
	register char *objName, *typeName;
	register Boolean async, objGood = false, typeGood = false, goAsync = false;
	register long objectPrefs = 0, typePrefs = 0;
	
	async = trapword & 0x0400;
	objName = ((char *)pb->NBPentityPtr)+9;
	typeName = objName + objName[0] + 1;

	if (objName && objName[0] && (objectPrefs = StringIsMaskable(objName)))				// should we worry about wiping the type?
	{
		if (registerEnabled(objectPrefs) && !IsStringMasked(objName))
		{
			MaskString(objName);
			goAsync = registerAsync(objectPrefs);
			objGood = true;
		}
	}

	if (typeName && typeName[0] && (typePrefs = StringIsMaskable(typeName)))				// should we worry about wiping the type?
	{
		if (registerEnabled(typePrefs) && !IsStringMasked(typeName))
		{
			MaskString(typeName);
			goAsync |= registerAsync(typePrefs);
			typeGood = true;
		}
	}

#if 1
	error = CallControl(trapword, pb);
#else
	// if the call is asynchronous, and the prefs say to go asynchrounous, we do async.
	// if the call is async, and the prefs say not to go async, we dont asymc
	// if the call isn't async, who cares?
	
	error = CallControl(async ? (goAsync ? trapword : trapword & 0x0400) : trapword, pb);

	if (async && !goAsync) pb->MPPioResult = error;
	
	if (objGood && !goAsync) RestoreString(objName);
	if (typeGood && !goAsync) RestoreString(typeName);
	
	if (!goAsync && async && pb->MPPioCompletion)
	{
		CallCompletion(trapword, pb, (long) pb->MPPioCompletion);
	}
#endif
	return error;
}

OSErr DoNewConfirm(short trapword, MPPPBPtr pb)
{
	register OSErr error;
	register char *objName, *typeName;
	register Boolean async, objGood = false, typeGood = false, goAsync = false;
	register long objectPrefs = 0, typePrefs = 0;
	
	async = trapword & 0x0400;
	objName = ((char *)pb->NBPentityPtr);
	typeName = objName + objName[0] + 1;
	
	if (objName && objName[0] && (objectPrefs = StringIsMaskable(objName)))
	{
		if (lookupEnabled(objectPrefs))
		{
			Munge(objName);
			goAsync = lookupAsync(objectPrefs);
			objGood = true;
		}
	}

	if (typeName && typeName[0] && (typePrefs = StringIsMaskable(typeName)))
	{
		if (lookupEnabled(typePrefs))
		{
			Munge(typeName);
			goAsync |= lookupAsync(typePrefs);
			typeGood = true;
		}
	}
#if 1
	error = CallControl(trapword, pb);
#else
	error = CallControl(async ? (goAsync ? trapword : trapword & 0x0400) : trapword, pb);

	if (async && !goAsync) pb->MPPioResult = error;
	
	if (objGood && !goAsync) RestoreString(objName);
	if (typeGood && !goAsync) RestoreString(typeName);
	
	if (!goAsync && async && pb->MPPioCompletion)
	{
		CallCompletion(trapword, pb, (long) pb->MPPioCompletion);
	}
#endif
	return error;
}
